@@ -7,7 +7,7 @@ import re
 from collections import OrderedDict
 from typing import List, Optional
 from types import SimpleNamespace
-from typing import Tuple
+from typing import Dict, Set, Tuple
 
 import pandas.io.sql
 
@@ -280,7 +280,6 @@ def _db_is_in_expr(dbmodel, expression):
     return "(" + is_in_expr + " IN " + x_expr + ")"
 
 
-# noinspection PyUnusedLocal
 def _db_count_expr(dbmodel, expression):
     """Count number of non-null entries (as in Pandas)"""
     if len(expression.args) != 1:
@@ -289,16 +288,6 @@ def _db_count_expr(dbmodel, expression):
     return f"SUM(CASE WHEN {e0} IS NOT NULL THEN 1 ELSE 0 END)"
 
 
-def _db_concat_expr(dbmodel, expression):
-    return (
-        "("  # TODO: cast each to char on way in
-        + " || ".join(
-            [dbmodel.expr_to_sql(ai, want_inline_parens=True) for ai in expression.args]
-        )
-        + ")"
-    )
-
-
 def _db_coalesce_expr(dbmodel, expression):
     return (
         "COALESCE("
@@ -312,6 +301,18 @@ def _db_coalesce_expr(dbmodel, expression):
     )
 
 
+def _db_pow_expr(dbmodel, expression):
+    if isinstance(expression.args[1], data_algebra.expr_rep.Value) and (expression.args[1].value == 1):
+        return dbmodel.expr_to_sql(expression.args[0], want_inline_parens=True)
+    return (
+        "POWER("
+        + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
+        + ", "
+        + dbmodel.expr_to_sql(expression.args[1], want_inline_parens=False)
+        + ")"
+    )
+
+
 def _db_round_expr(dbmodel, expression):
     return (
         "ROUND("
@@ -320,6 +321,14 @@ def _db_round_expr(dbmodel, expression):
     )
 
 
+def _db_around_expr(dbmodel, expression):
+    if isinstance(expression.args[1], data_algebra.expr_rep.Value) and (expression.args[1].value == 0):
+        return dbmodel.expr_to_sql(expression.args[0].round(), want_inline_parens=False)
+    mult = data_algebra.expr_rep.Value(10.0).__pow__(expression.args[1])
+    derived = (expression.args[0] * mult).round() / mult
+    return dbmodel.expr_to_sql(derived, want_inline_parens=True)
+
+
 def _db_floor_expr(dbmodel, expression):
     return (
         "FLOOR("
@@ -337,23 +346,9 @@ def _db_ceil_expr(dbmodel, expression):
 
 
 def _db_int_divide_expr(dbmodel, expression):
-    return (
-        "FLOOR("
-        + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=True)
-        + " / "
-        + dbmodel.expr_to_sql(expression.args[1], want_inline_parens=True)
-        + ")"
-    )
-
-
-def _db_pow_expr(dbmodel, expression):
-    return (
-        "POWER("
-        + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
-        + ", "
-        + dbmodel.expr_to_sql(expression.args[1], want_inline_parens=False)
-        + ")"
-    )
+    # example of a derived expression
+    ratio = (expression.args[0] / expression.args[1]).floor()
+    return dbmodel.expr_to_sql(ratio, want_inline_parens=False)
 
 
 def _db_nunique_expr(dbmodel, expression):
@@ -382,6 +377,16 @@ def _as_str(dbmodel, expression):
     )
 
 
+def _db_concat_expr(dbmodel, expression):
+    return (
+        "("
+        + " || ".join(
+            [dbmodel.expr_to_sql(ai.as_str(), want_inline_parens=True) for ai in expression.args]
+        )
+        + ")"
+    )
+
+
 def _trimstr(dbmodel, expression):
     return (
         "SUBSTR("
@@ -394,6 +399,18 @@ def _trimstr(dbmodel, expression):
     )
 
 
+def _any_expr(dbmodel, expression):
+    derived = expression.args[0].max() >= data_algebra.expr_rep.Value(1)
+    return dbmodel.expr_to_sql(derived, want_inline_parens=True)
+
+
+def _all_expr(dbmodel, expression):
+    derived = expression.args[0].min() >= data_algebra.expr_rep.Value(1)
+    return dbmodel.expr_to_sql(derived, want_inline_parens=True)
+
+
+# date/time fns
+
 def _datetime_to_date(dbmodel, expression):
     return (
         "DATE("
@@ -539,6 +556,7 @@ db_expr_formatters = {
     "concat": _db_concat_expr,
     "coalesce": _db_coalesce_expr,
     "round": _db_round_expr,
+    "around": _db_around_expr,
     "floor": _db_floor_expr,
     "ceil": _db_ceil_expr,
     "//": _db_int_divide_expr,
@@ -549,6 +567,8 @@ db_expr_formatters = {
     "as_int64": _as_int64,
     "as_str": _as_str,
     "trimstr": _trimstr,
+    "any": _any_expr,
+    "all": _all_expr,
     "datetime_to_date": _datetime_to_date,
     "parse_datetime": _parse_datetime,
     "parse_date": _parse_date,
@@ -569,7 +589,7 @@ db_expr_formatters = {
 
 db_default_op_replacements = {
     "==": "=",
-    "cumsum": "sum",
+    "cumsum": "SUM",
     "and": "AND",
     "&": "AND",
     "&&": "AND",
@@ -578,6 +598,12 @@ db_default_op_replacements = {
     "||": "OR",
     "!": "NOT",
     "~": "NOT",
+    "_count": "COUNT",
+    "_ngroup": "NGROUP",
+    "_row_number": "ROW_NUMBER",
+    "_size": "SIZE",
+    "_connected_components": "CONNECTED_COMPONENTS",
+    "_uniform": "RAND",
 }
 
 
@@ -883,8 +909,12 @@ class DBModel:
             op = expression.op
             if op in self.op_replacements.keys():
                 op = self.op_replacements[op]
+            elif op.lower() in  self.op_replacements.keys():
+                op = self.op_replacements[op.lower()]
             if op in self.sql_formatters.keys():
                 return self.sql_formatters[op](self, expression)
+            if op.lower() in self.sql_formatters.keys():
+                return self.sql_formatters[op.lower()](self, expression)
             if (len(expression.args) > 1) and expression.inline:
                 subs = [
                     self.expr_to_sql(ai, want_inline_parens=True)
@@ -966,7 +996,7 @@ class DBModel:
                 terms[k] = k  # these get quoted later
             view_name = "table_reference_" + str(temp_id_source[0])
             temp_id_source[0] = temp_id_source[0] + 1
-            near_sql = data_algebra.near_sql.NearSQLUnaryStep(
+            return data_algebra.near_sql.NearSQLUnaryStep(
                 terms=terms,
                 query_name=view_name,
                 quoted_query_name=self.quote_identifier(view_name),
@@ -997,7 +1027,7 @@ class DBModel:
                 subops[k] = op
         if len(subops) <= 0:
             # using was not None is this case as len(extend_node.ops)>0 and all keys are in extend_node.column_names
-            return extend_node.sources[0].to_near_sql_implementation(
+            return extend_node.sources[0].to_near_sql_implementation_(
                 db_model=self, using=using, temp_id_source=temp_id_source
             )
         if len(using) < 1:
@@ -1007,7 +1037,7 @@ class DBModel:
             raise KeyError("referred to unknown columns: " + str(missing))
         # get set of columns we need from subquery
         subusing = extend_node.columns_used_from_sources(using=using)[0]
-        subsql = extend_node.sources[0].to_near_sql_implementation(
+        subsql = extend_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         window_term = ""
@@ -1031,7 +1061,7 @@ class DBModel:
                 window_term = window_term + "ORDER BY " + ", ".join(rt) + " "
                 window_vars.update(extend_node.order_by)
             window_term = window_term + " ) "
-        terms = OrderedDict()
+        terms: Dict[str, Optional[str]] = OrderedDict()
         declared_term_dependencies = OrderedDict()
         origcols = [k for k in using if k not in subops.keys()]
         for k in origcols:
@@ -1039,12 +1069,12 @@ class DBModel:
             declared_term_dependencies[k] = {k}
         for (ci, oi) in subops.items():
             terms[ci] = self.expr_to_sql(oi) + window_term
-            cols_used_in_term = set()
+            cols_used_in_term: Set[str] = set()
             oi.get_column_names(cols_used_in_term)
             cols_used_in_term.update(window_vars)
             declared_term_dependencies[ci] = cols_used_in_term
         annotation = str(
-            extend_node.to_python_implementation(print_sources=False, indent=-1)
+            extend_node.to_python_src_(print_sources=False, indent=-1)
         )
         # TODO: see if we can merge with subsql instead of building a new one
         if (
@@ -1084,7 +1114,11 @@ class DBModel:
             )
             if len(contention) == 0:
                 # merge our stuff into subsql
-                subsql.annotation = subsql.annotation + "." + annotation
+                assert subsql.terms is not None  # type hint
+                if subsql.annotation is None:
+                    subsql.annotation = annotation
+                else:
+                    subsql.annotation = subsql.annotation + "." + annotation
                 for k in our_non_trivial_terms:
                     subsql.terms[k] = terms[k]
                     subsql.declared_term_dependencies[k] = declared_term_dependencies[k]
@@ -1131,7 +1165,7 @@ class DBModel:
         subusing = project_node.columns_used_from_sources(using=using)[0]
         terms = {ci: self.expr_to_sql(oi) for (ci, oi) in subops.items()}
         terms.update({g: None for g in project_node.group_by})
-        subsql = project_node.sources[0].to_near_sql_implementation(
+        subsql = project_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         view_name = "project_" + str(temp_id_source[0])
@@ -1149,7 +1183,7 @@ class DBModel:
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
             annotation=str(
-                project_node.to_python_implementation(print_sources=False, indent=-1)
+                project_node.to_python_src_(print_sources=False, indent=-1)
             ),
         )
         return near_sql
@@ -1177,7 +1211,7 @@ class DBModel:
         if using is None:
             using = OrderedSet(select_rows_node.column_names)
         subusing = select_rows_node.columns_used_from_sources(using=using)[0]
-        subsql = select_rows_node.sources[0].to_near_sql_implementation(
+        subsql = select_rows_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         view_name = "select_rows_" + str(temp_id_source[0])
@@ -1193,7 +1227,7 @@ class DBModel:
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
             annotation=str(
-                select_rows_node.to_python_implementation(
+                select_rows_node.to_python_src_(
                     print_sources=False, indent=-1
                 )
             ),
@@ -1223,7 +1257,7 @@ class DBModel:
         subusing = [
             c for c in select_columns_node.column_selection if c in subusing
         ]  # fix order
-        subsql = select_columns_node.sources[0].to_near_sql_implementation(
+        subsql = select_columns_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=set(subusing), temp_id_source=temp_id_source
         )
         # order/limit columns
@@ -1257,7 +1291,7 @@ class DBModel:
         if using is None:
             using = OrderedSet(drop_columns_node.column_names)
         subusing = drop_columns_node.columns_used_from_sources(using=using)[0]
-        subsql = drop_columns_node.sources[0].to_near_sql_implementation(
+        subsql = drop_columns_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         # /limit columns
@@ -1286,7 +1320,7 @@ class DBModel:
             using_was_None = True
         subusing = order_node.columns_used_from_sources(using=using)[0]
         subusing = [c for c in order_node.column_names if c in subusing]  # fix order
-        subsql = order_node.sources[0].to_near_sql_implementation(
+        subsql = order_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=set(subusing), temp_id_source=temp_id_source
         )
         view_name = "order_rows_" + str(temp_id_source[0])
@@ -1294,7 +1328,7 @@ class DBModel:
         terms = None
         if not using_was_None:
             terms = {ci: None for ci in subusing}
-        suffix = []
+        suffix: List[str] = []
         if len(order_node.order_columns) > 0:
             suffix = (
                 suffix
@@ -1317,7 +1351,7 @@ class DBModel:
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             suffix=suffix,
             annotation=str(
-                order_node.to_python_implementation(print_sources=False, indent=-1)
+                order_node.to_python_src_(print_sources=False, indent=-1)
             ),
         )
         return near_sql
@@ -1337,7 +1371,7 @@ class DBModel:
         if using is None:
             using = OrderedSet(rename_node.column_names)
         subusing = rename_node.columns_used_from_sources(using=using)[0]
-        subsql = rename_node.sources[0].to_near_sql_implementation(
+        subsql = rename_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=subusing, temp_id_source=temp_id_source
         )
         view_name = "rename_" + str(temp_id_source[0])
@@ -1356,7 +1390,7 @@ class DBModel:
             quoted_query_name=self.quote_identifier(view_name),
             sub_sql=subsql.to_bound_near_sql(columns=subusing),
             annotation=str(
-                rename_node.to_python_implementation(print_sources=False, indent=-1)
+                rename_node.to_python_src_(print_sources=False, indent=-1)
             ),
         )
         return near_sql
@@ -1401,10 +1435,10 @@ class DBModel:
         using_left, using_right = join_node.columns_used_from_sources(
             using=using.union(by_set)
         )
-        sql_left = join_node.sources[0].to_near_sql_implementation(
+        sql_left = join_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=using_left, temp_id_source=temp_id_source
         )
-        sql_right = join_node.sources[1].to_near_sql_implementation(
+        sql_right = join_node.sources[1].to_near_sql_implementation_(
             db_model=self, using=using_right, temp_id_source=temp_id_source
         )
         return using_left, sql_left, using_right, sql_right
@@ -1472,7 +1506,7 @@ class DBModel:
             sub_sql2=sql_right.to_bound_near_sql(columns=using_right, force_sql=False),
             suffix=on_terms,
             annotation=str(
-                join_node.to_python_implementation(print_sources=False, indent=-1)
+                join_node.to_python_src_(print_sources=False, indent=-1)
             ),
         )
         return near_sql
@@ -1501,10 +1535,10 @@ class DBModel:
         using_right = subusing[1]
         if set(using_left) != set(using_right):
             raise ValueError("left/right usings did not match")
-        sql_left = concat_node.sources[0].to_near_sql_implementation(
+        sql_left = concat_node.sources[0].to_near_sql_implementation_(
             db_model=self, using=using_left, temp_id_source=temp_id_source
         )
-        sql_right = concat_node.sources[1].to_near_sql_implementation(
+        sql_right = concat_node.sources[1].to_near_sql_implementation_(
             db_model=self, using=using_left, temp_id_source=temp_id_source
         )
         view_name = "concat_rows_" + str(temp_id_source[0])
@@ -1532,7 +1566,7 @@ class DBModel:
                 columns=using_left, force_sql=True, constants=constants_right,
             ),
             annotation=str(
-                concat_node.to_python_implementation(print_sources=False, indent=-1)
+                concat_node.to_python_src_(print_sources=False, indent=-1)
             ),
         )
         return near_sql
@@ -1548,7 +1582,7 @@ class DBModel:
         assert isinstance(sql_format_options, SQLFormatOptions)
         ops.columns_used()  # for table consistency check/raise
         temp_id_source = [0]
-        near_sql = ops.to_near_sql_implementation(
+        near_sql = ops.to_near_sql_implementation_(
             db_model=self, using=None, temp_id_source=temp_id_source
         )
         assert isinstance(near_sql, data_algebra.near_sql.NearSQL)
@@ -1558,7 +1592,7 @@ class DBModel:
             len_sequence = len(sequence.previous_steps)
             # can fall back to the non-with path
             if len(sequence.previous_steps) >= 1:
-                sql_sequence = []
+                sql_sequence: List[str] = []
                 for i in range(len_sequence):
                     nmi = sequence.previous_steps[i][0]  # already quoted
                     sqli = sequence.previous_steps[i][1].to_sql_str_list(
@@ -1894,7 +1928,9 @@ class DBModel:
             and (near_sql.annotation is not None)
             and (len(near_sql.annotation) > 0)
         ):
-            sql_start = "SELECT  -- " + _clean_annotation(near_sql.annotation)
+            clean_anno = _clean_annotation(near_sql.annotation)
+            if clean_anno is not None:
+                sql_start = "SELECT  -- " + clean_anno
         sql = (
             [sql_start]
             + self._indent_and_sep_terms(
@@ -1922,9 +1958,11 @@ class DBModel:
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
         assert isinstance(sql_format_options, SQLFormatOptions)
-        sql = []
+        sql: List[str] = []
         if near_sql.annotation is not None:
-            sql = sql + ["-- " + _clean_annotation(near_sql.annotation)]
+            clean_anno = _clean_annotation(near_sql.annotation)
+            if clean_anno is not None:
+                sql = sql + ["-- " + clean_anno]
         if add_select:
             sql = sql + ["SELECT"]
         sql = sql + [" " + v for v in near_sql.prefix]
@@ -1956,9 +1994,12 @@ class DBModel:
         if sql_format_options is None:
             sql_format_options = self.default_SQL_format_options
         assert isinstance(sql_format_options, SQLFormatOptions)
+        if near_sql.terms is None:
+            terms: Dict[str, Optional[str]] = OrderedDict()
+        else:
+            terms = near_sql.terms
         if columns is None:
-            columns = [k for k in near_sql.terms.keys()]
-        terms = near_sql.terms
+            columns = [k for k in terms.keys()]
         if (constants is not None) and (len(constants) > 0):
             terms.update(constants)
         terms_strs = [self.enc_term_(k, terms=terms) for k in columns]
@@ -1971,7 +2012,9 @@ class DBModel:
             and (near_sql.annotation is not None)
             and (len(near_sql.annotation) > 0)
         ):
-            sql_start = "SELECT  -- " + _clean_annotation(near_sql.annotation)
+            clean_anno = _clean_annotation(near_sql.annotation)
+            if clean_anno is not None:
+                sql_start = "SELECT  -- " + clean_anno
         if is_union:
             substr_1 = near_sql.sub_sql1.to_sql_str_list(
                 db_model=self, sql_format_options=sql_format_options
@@ -2063,7 +2106,7 @@ class DBHandle:
 
     def execute(self, q) -> None:
         """
-        Execute a SQL query.
+        Execute a SQL query or operator dag.
         """
         self.db_model.execute(conn=self.conn, q=q)
 
