@@ -278,6 +278,9 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
     def data_frame(self, arg=None):
         """
         Build a new emtpy data frame.
+
+        :param arg" optional argument passed to constructor.
+        :return: data frame
         """
         if arg is None:
             # noinspection PyUnresolvedReferences
@@ -444,16 +447,14 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             or (len(op.order_by) > 0)
         )
         if window_situation:
-            op.check_extend_window_fns()
-        res = op.sources[0].eval_implementation(
+            op.check_extend_window_fns_()
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         if not window_situation:
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore")  # out of range things like arccosh were warning
                 new_cols = {k: opk.evaluate(res) for k, opk in op.ops.items()}
-            # for k, v in new_cols.items():
-            #     res[k] = v
             new_frame = self.columns_to_frame_(new_cols, target_rows=res.shape[0])
             res = self.add_data_frame_columns_to_data_frame_(res, new_frame)
         else:
@@ -504,19 +505,25 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             # perform calculations
             for (k, opk) in op.ops.items():
                 # work on a slice of the data frame
-                # TODO: document exactly which of these are available
-                # TODO: check if need any of these in project
+                # Availability roughly documented in:
+                # https://github.com/WinVector/data_algebra/blob/main/Examples/Methods/data_algebra_catalog.ipynb
+                # (essentially none but _ngroup)
                 if len(opk.args) <= 0:
-                    if opk.op in {"row_number", "_row_number", "count", "_count"}:
+                    # check for and remove initial underbar
+                    assert isinstance(opk.op, str)
+                    assert len(opk.op) > 1
+                    assert opk.op[0] == '_'
+                    zero_op = opk.op[1:]
+                    if zero_op in {"row_number", "count"}:
                         subframe[k] = opframe.cumcount() + 1
-                    elif opk.op in {"ngroup", "_ngroup"}:
+                    elif zero_op in {"ngroup"}:
                         subframe[k] = opframe.ngroup()
-                    elif opk.op in {"size", "_size"}:
+                    elif zero_op in {"size"}:
                         subframe[k] = opframe[standin_name].transform(
-                            opk.op
+                            zero_op
                         )  # Pandas transform, not data_algebra
                     else:
-                        raise KeyError("not implemented: " + str(k) + ": " + str(opk))
+                        raise KeyError("not implemented in windowed situation: " + str(k) + ": " + str(opk))
                 else:
                     transform_args = []
                     if len(opk.args) > 1:
@@ -561,7 +568,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         # try the following tutorial:
         # https://www.shanelynn.ie/summarising-aggregation-and-grouping-data-in-python-pandas/
         data_algebra_temp_cols = {}
-        res = op.sources[0].eval_implementation(
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         for (k, opk) in op.ops.items():
@@ -601,7 +608,11 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
                 if len(opk.args) > 0:
                     vk = res[value_name].agg(opk.op)
                 else:
-                    vk = res["_data_table_temp_col"].agg(opk.op)
+                    # expect and strip off initial underbar
+                    assert isinstance(opk.op, str)
+                    assert len(opk.op) > 1
+                    assert opk.op[0] == '_'
+                    vk = res["_data_table_temp_col"].agg(opk.op[1:])
                 cols[k] = vk
         else:
             cols = {"_data_table_temp_col": res["_data_table_temp_col"].agg("sum")}
@@ -627,7 +638,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.SelectRowsNode"
             )
-        res = op.sources[0].eval_implementation(
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         if res.shape[0] < 1:
@@ -644,7 +655,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.SelectColumnsNode"
             )
-        res = op.sources[0].eval_implementation(
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         return res[op.column_selection]
@@ -657,7 +668,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.DropColumnsNode"
             )
-        res = op.sources[0].eval_implementation(
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         column_selection = [c for c in res.columns if c not in op.column_deletions]
@@ -671,7 +682,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.OrderRowsNode"
             )
-        res = op.sources[0].eval_implementation(
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         if res.shape[0] > 1:
@@ -693,15 +704,15 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.RenameColumnsNode"
             )
-        res = op.sources[0].eval_implementation(
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         return res.rename(columns=op.reverse_mapping)
 
     # noinspection PyMethodMayBeStatic
-    def standardize_join_code(self, jointype):
+    def standardize_join_code_(self, jointype):
         """
-        Map join names to Pandas names.
+        Map join names to Pandas names. Internal method.
         """
         assert isinstance(jointype, str)
         jointype = jointype.lower()
@@ -723,10 +734,10 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.NaturalJoinNode"
             )
-        left = op.sources[0].eval_implementation(
+        left = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
-        right = op.sources[1].eval_implementation(
+        right = op.sources[1].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         common_cols = set([c for c in left.columns]).intersection(
@@ -750,7 +761,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
         res = self.pd.merge(
             left=left,
             right=right,
-            how=self.standardize_join_code(op.jointype),
+            how=self.standardize_join_code_(op.jointype),
             on=by,
             sort=False,
             suffixes=("", "_tmp_right_col"),
@@ -774,10 +785,10 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.ConcatRowsNode"
             )
-        left = op.sources[0].eval_implementation(
+        left = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
-        right = op.sources[1].eval_implementation(
+        right = op.sources[1].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         if op.id_column is not None:
@@ -809,7 +820,7 @@ class PandasModelBase(data_algebra.data_model.DataModel, ABC):
             raise TypeError(
                 "op was supposed to be a data_algebra.data_ops.ConvertRecordsNode"
             )
-        res = op.sources[0].eval_implementation(
+        res = op.sources[0].eval_implementation_(
             data_map=data_map, data_model=self, narrow=narrow
         )
         return op.record_map.transform(res, local_data_model=self)
