@@ -8,6 +8,7 @@ import data_algebra
 
 import pickle
 import hashlib
+import traceback
 
 import data_algebra.db_model
 import data_algebra.SQLite
@@ -31,13 +32,9 @@ run_direct_ops_path_tests = False
 global_test_result_cache = None
 
 
-def formats_to_self(ops) -> bool:
+def re_parse(ops):
     """
-    Check a operator dag formats and parses back to itself.
-    Can raise exceptions. Also checks pickling.
-
-    :param ops: data_algebra.data_ops.ViewRepresentation
-    :return: logical, True if formats and evals back to self
+    Return copy of object made by dumpint to string via repr() and then evaluating that string.
     """
     str1 = repr(ops)
     ops2 = eval(
@@ -47,8 +44,18 @@ def formats_to_self(ops) -> bool:
             "pd": data_algebra.default_data_model.pd
         },  # make our definition of pandas available
     )
-    # str2 = repr(ops2)
-    # strings_match = str1 == str2  # probably too strict
+    return ops2
+
+
+def formats_to_self(ops) -> bool:
+    """
+    Check a operator dag formats and parses back to itself.
+    Can raise exceptions. Also checks pickling.
+
+    :param ops: data_algebra.data_ops.ViewRepresentation
+    :return: logical, True if formats and evals back to self
+    """
+    ops2 = re_parse(ops)
     ops_match = ops == ops2
     assert ops_match
     pickle_string = pickle.dumps(ops)
@@ -120,12 +127,28 @@ def equivalent_frames(
             if not numpy.all(ca_null):
                 ca = ca[ca_null == False]
                 cb = cb[cb_null == False]
-                if local_data_model.can_convert_col_to_numeric(ca) != local_data_model.can_convert_col_to_numeric(cb):
+                ca_can_be_numeric = False
+                ca_n = numpy.asarray([0.0])  # just a typing hint
+                # noinspection PyBroadException
+                try:
+                    ca_n = numpy.asarray(ca, dtype=float)
+                    ca_can_be_numeric = True
+                except Exception:
+                    pass
+                cb_can_be_numeric = False
+                cb_n = numpy.asarray([0.0])   # just a typing hint
+                # noinspection PyBroadException
+                try:
+                    cb_n = numpy.asarray(cb, dtype=float)
+                    cb_can_be_numeric = True
+                except Exception:
+                    pass
+                if ca_can_be_numeric != cb_can_be_numeric:
                     return False
-                if local_data_model.can_convert_col_to_numeric(a[c]):
-                    ca = numpy.asarray(ca, dtype=float)
-                    cb = numpy.asarray(cb, dtype=float)
-                    dif = numpy.abs(ca - cb) / numpy.maximum(numpy.maximum(ca, cb), 1)
+                if ca_can_be_numeric and cb_can_be_numeric:
+                    if len(ca_n) != len(cb_n):
+                        return False
+                    dif = numpy.abs(ca_n - cb_n) / numpy.maximum(numpy.maximum(numpy.abs(ca_n), numpy.abs(cb_n)), 1.0)
                     if numpy.max(dif) > float_tol:
                         return False
                 else:
@@ -208,13 +231,14 @@ def _run_handle_experiments(
                 assert res_db_ops is not None
             for i in range(len(sql_statements)):
                 if res_db_sql[i] is None:
-                    res_db_sql[i] = db_handle.read_query(sql_statements[i])
+                    res_db_sql_i = db_handle.read_query(sql_statements[i])
+                    res_db_sql[i] = res_db_sql_i
                     if (
                         alter_cache
                         and (test_result_cache is not None)
-                        and (res_db_sql[i] is not None)
+                        and (res_db_sql_i is not None)
                     ):
-                        test_result_cache[mk_key(i)] = res_db_sql[i].copy()
+                        test_result_cache[mk_key(i)] = res_db_sql_i.copy()
         except Exception as e:
             caught = e
         for k in to_del:
@@ -298,11 +322,8 @@ def check_transform_on_handles(
     for k in cols_used.keys():
         v = data[k]
         assert local_data_model.is_appropriate_data_instance(v)
-    if check_parse:
-        if not formats_to_self(ops):
-            raise ValueError("ops did not round-trip format")
-    res = ops.eval(data_map=data)
     # try pandas path
+    res = ops.eval(data_map=data, check_incoming_data_constraints=True)
     if not local_data_model.is_appropriate_data_instance(res):
         raise ValueError(
             "expected res to be local_data_model.pd.DataFrame, got: " + str(type(res))
@@ -316,8 +337,26 @@ def check_transform_on_handles(
         check_row_order=check_row_order,
     ):
         raise ValueError("Pandas eval result did not match expect")
+    if check_parse:
+        if not formats_to_self(ops):
+            raise ValueError("ops did not round-trip format")
+        ops_2 = re_parse(ops)
+        res_2 = ops_2.eval(data_map=data, check_incoming_data_constraints=True)
+        if not local_data_model.is_appropriate_data_instance(res_2):
+            raise ValueError(
+                "(reparse) expected res to be local_data_model.pd.DataFrame, got: " + str(type(res_2))
+            )
+        if not equivalent_frames(
+            res_2,
+            expect,
+            float_tol=float_tol,
+            check_column_order=check_column_order,
+            cols_case_sensitive=cols_case_sensitive,
+            check_row_order=check_row_order,
+        ):
+            raise ValueError("(reparse) Pandas eval result did not match expect")
     if len(data) == 1:
-        res_t = ops.transform(list(data.values())[0])
+        res_t = ops.transform(list(data.values())[0], check_incoming_data_constraints=True)
         if not equivalent_frames(
             res_t,
             expect,
@@ -491,6 +530,8 @@ def check_transform(
             db_handles=db_handles,
             empty_produces_empty=empty_produces_empty,
         )
+    except AssertionError:
+        traceback.print_exc()
     except Exception as exc:
         caught = exc
 
